import csv
import shutil
from pathlib import Path
from sys import argv
import os
from argparse import ArgumentParser
import json
import re
import yaml
from os import listdir


def csv_calc(filename):
    """
    Evaluates the expression at each line and add the result as the fourth column.
    Finally, it stores the csv file with the results to the file
    :param filename: The filename with the input data and also the output
    :return: nothing
    """
    data_path = Path(filename)

    with data_path.open() as data_file:
        reader = csv.reader(data_file, delimiter=';')

        result_lines = list()

        for line in reader:
            result_line = list()
            result_line.extend(line)

            if line[2] in '+-*/':
                '''
                Checking the legitimacy of the operation is very important. Note that using *eval* is generally very 
                risky, since a hacker can inject and execute any command. Try, e.g., a line: 1;1;+ os.system('ls -al') +
                '''
                result_line.append(eval(line[0] + line[2] + line[1]))

            else:
                result_line.append('Operation not supported')

            result_lines.append(result_line)

        output_path = Path(filename).with_suffix('.out.csv')

        with output_path.open(mode='w') as output_file:
            writer = csv.writer(output_file, delimiter=';')
            writer.writerows(result_lines)


def create_json_file(args):
    """
    Create a json file containing data from arguments.
    :param args: the commandline, for details, look below for the args parsing
    :return: nothing
    """
    parser = ArgumentParser()

    # --id --firstname --lastname --age
    parser.add_argument('--id', type=int, required=True)
    parser.add_argument('--firstname', type=str, required=False, default='')
    parser.add_argument('--lastname', type=str, required=False, default='')
    parser.add_argument('--age', type=int, required=False, default=0)
    parser.add_argument('--force', action='store_true', default=False)

    data = parser.parse_args(args)

    filename = str(data.id) + '.json'

    if (not Path(filename).exists()) or data.force:

        with open(filename, mode='wt') as file:
            data.__delattr__('force')
            json.dump(data.__dict__, file)
    else:
        print('ERROR: File already exists.')


def jsonl2csv(filename):
    """
    Loads a jsonl file and outputs a corresponding csv file (same filename, just csv extension)
    :param filename: the input file name containing in the JSONL format
    :return: nothing
    """
    with open(filename, mode="rt") as file:
        lines = file.readlines()
        data = list()
        for line in lines:
            data.append(json.loads(line))

        with Path(filename).with_suffix('.csv').open(mode='wt') as f:
            writer = csv.writer(f, delimiter=';')
            writer.writerow(['id', 'first name', 'last name', 'age'])
            values = [items.values() for items in data]
            writer.writerows(values)


def mails(filename):
    """
    Reads the names and emails from a file and create a yaml file with the data
    :param filename: the filename containing the input data
    :return: nothing
    """
    with open(filename, mode='rt') as f:
        people = dict()
        people['people'] = list()
        for record in f.readlines():
            match = re.match('([a-zA-z ]*)<(.*)>', record.strip('\n'))
            if match:
                name = match.group(1).strip(' ') if match.group(1) != '' else 'N/A'
                email = match.group(2)
            else:  # only email
                name = 'N/A'
                email = record.strip('\n')

            people['people'].append({'name': name, 'email': email})

        with Path(filename).with_suffix('.yaml').open(mode='wt') as out:
            # This will create a file with the email first, then the name, as this is sorted in dict.
            # Do not forget about the default flow style, otherwise the format will be different.
            yaml.dump(people, out, default_flow_style=False)


def my_shell():
    """
    A simple shell supporting the commands below.
    Further commands can be easily added.
    :return: nothing
    """

    cmd = input('Your command, sir: ')
    cmd = cmd.strip()
    while cmd != 'exit':
        parts = cmd.split(' ')

        if parts[0] == 'ls':
            listing = sorted([filename for filename in listdir('.')])

            for filename in listing:
                print(filename)

        elif parts[0] == 'pwd':
            print(os.getcwd())

        elif parts[0] == 'cd':
            if len(parts) == 2:
                try:
                    os.chdir(parts[1])
                except FileNotFoundError:
                    print('The specified dir does not exist.')
            else:
                print('New dir expected.')

        elif parts[0] == 'cp':
            if len(parts) == 3:
                try:
                    shutil.copy(parts[1], parts[2])
                except FileNotFoundError:
                    print('Error copying file. Does the source exist? Does the destination dir exist?')
            else:
                print('Source and target expected.')

        elif parts[0] == 'rm':
            if len(parts) == 2:
                try:
                    if input('Do you really want to delete file ' + parts[1] + '? ').upper()[0] == 'Y':
                        os.remove(parts[1])
                except OSError as e:
                    print('Error removing file: ' + e.strerror)

        elif parts[0] == 'rmdir':
            if len(parts) == 2:
                try:
                    if input('Do you really want to delete the directory ' + parts[1] + ' and all files inside? ').upper()[0] == 'Y':
                        shutil.rmtree(parts[1])
                except OSError as e:
                    print('Error removing directory: ' + e.strerror)

        else:
            print('Unknown command.')

        cmd = input('Your command, sir: ').strip()


# CSV
#if len(argv) == 2:
#    csv_calc(argv[1])
#csv_calc('input.csv')

# JSON 1
#create_json_file(['--id=1234', '--firstname=Ayrton', '--lastname=Senna', '--age=25'])

#jsonl to csv
#jsonl2csv('1234.jsonl')

# create yaml files with emails
# mails('emails.txt')

# simple shell
#my_shell()



